#ifndef CALCINTGRT_CK
#define CALCINTGRT_CK
//
//		Ver. 2014.09
//

namespace myLib {

	using namespace System;

	template<class TF>		//		TF: Class type for the functor
	class TAdaptiveGL {
		public:
			double acc;
			TF & f;			//		Functor:  One independent variable.

			TAdaptiveGL(TF & vf, double vacc = 1.0e-14) : f(vf) { acc = vacc; }
			virtual ~TAdaptiveGL(){}

			double integral(double a, double b);
			void   calcIntegral(double & s0, double a, double b);
			double Gauss_Legendre(double a, double b);
	};
	template<class TF>
	double TAdaptiveGL<TF>::integral(double a, double b){
		double v = 0.0;
		calcIntegral(v, a, b);
		return v;
	}

	template<class TF>
	void TAdaptiveGL<TF>::calcIntegral(double & s0, double a, double b){
		double s1 = Gauss_Legendre(a, 0.5*(a + b));
		double s2 = Gauss_Legendre(0.5*(a + b), b);
		if (((Math::Abs(s0 - s1 - s2) < (acc * Math::Abs(s1 + s2)))
			||
			((Math::Abs(s0) + Math::Abs(s1 + s2)) < acc))
			&&
			(Math::Abs(b - a) < 1.0)){
			s0 = s1 + s2;
			return;
		}
		else{
			calcIntegral(s1, a, 0.5*(a + b));
			calcIntegral(s2, 0.5*(a + b), b);
			s0 = s1 + s2;
			return;
		}
	}

	template<class TF>
	double TAdaptiveGL<TF>::Gauss_Legendre(double a, double b){
		double x0[9] = {
			-9.68160239507626090E-0001,
			-8.36031107326635794E-0001,
			-6.13371432700590397E-0001,
			-3.24253423403808929E-0001,
			0.00000000000000000E+0000,
			3.24253423403808929E-0001,
			6.13371432700590397E-0001,
			8.36031107326635794E-0001,
			9.68160239507626090E-0001
		};
		double w[9] = {
			8.12743883615744120E-0002,
			1.80648160694857404E-0001,
			2.60610696402935462E-0001,
			3.12347077040002840E-0001,
			3.30239355001259763E-0001,
			3.12347077040002840E-0001,
			2.60610696402935462E-0001,
			1.80648160694857404E-0001,
			8.12743883615744120E-0002
		};

		double v1 = 0.0;
		double i_w = b - a;
		double xi;
		for (int i = 0; i < 9; i++){
			xi = (0.5 * i_w * x0[i]) + (0.5 * (b + a));
			v1 += w[i] * f(xi);
		}
		v1 = 0.5 * i_w * v1;
		return v1;
	}


}  //  myLib

#endif
