//			This program was written in Visual C++ 2013
//			     by Yasuharu okamoto, 2015

#ifndef CALC_T_CK
#define CALC_T_CK

#include "stdafx.h"
#include "CalcIntgrt.h"

using namespace myLib;

namespace calc_t {

	extern double sqr(double v);
	
	class Ct0 {
	public:
		double df;
		double operator()(double x){
			double v = 1.0 / (1.0 + sqr(x) / df);
			return Math::Pow(v, (df + 1.0) / 2.0);
		}
	};

	class Ct0_tan{
	public:
		double df;
		Ct0 t0;
		double operator()(double u){
			return t0(Math::Tan(u)) / sqr(Math::Cos(u));
		}
	};

	class CCum_t {
	public:
		double df, coeff;
		Ct0 t0;
		Ct0_tan t0_tan;

		CCum_t(){}
		CCum_t(double vdf) : df(vdf) {
			t0.df = df;
			t0_tan.df = df;
			t0_tan.t0.df = df;
			TAdaptiveGL<Ct0_tan> gl(t0_tan);
			double v = gl.integral(-Math::PI / 2.0, Math::PI / 2.0);
			coeff = 1.0 / v;
		}
		void set_df(double vdf){
			df = vdf;
			t0.df = df;
			t0_tan.df = df;
			t0_tan.t0.df = df;
			TAdaptiveGL<Ct0_tan> gl(t0_tan);
			double v = gl.integral(-Math::PI / 2.0, Math::PI / 2.0);
			coeff = 1.0 / v;
		}

		double operator()(double t){
			if (t == 0.0) {
				return 0.5;
			}
			else {
				TAdaptiveGL<Ct0> gl(t0);
				if (t > 0.0) {
					double v = gl.integral(0.0, t);
					v = 0.5 + v * coeff;
					return v;
				}
				else {
					double v = gl.integral(t, 0.0);
					v = 0.5 - v * coeff;
					return v;
				};
			}
		}

	};

	//     Bisection@ɂ鍪̌vZ    
	template <class TF>
	void Bisection(
		TF & f,						 //	  ֐IuWFNg
		double s,					 //   f֐̂Ƃ s > 0.0
		double c,					 //   ɑ΂֐l
		double L_b, double U_b,		 //   ̑݋Ԃ̉Ə
		double & Root,				 //     f(Root) = c
		double acc){
		if ((f(L_b) - c) * (f(U_b) - c) > 0.0){
			throw  gcnew Exception("Ɖ̑gݍ킹sK؂łB");
		}
		do {
			double m = 0.5 * (L_b + U_b);
			double v = f(m);
			if (s * (v - c) > 0.0) {
				U_b = m;
			}
			else {
				L_b = m;
			}
		} while ((Math::Abs(U_b - L_b) >= acc * Math::Abs(U_b))
			&&
			((Math::Abs(U_b) + Math::Abs(L_b)) >= 1.0e-14)
			);
		Root = 0.5 * (L_b + U_b);
	}


	//     Newton@ɂ鍪̌vZ   
	template <class TF, class TDF>
	void Newton(
		double & x,						//	l
		double c,						//  ֐l
		TF & f,							//  ֐
		TDF & df,						//  ֐
		double acc){
		int NStep = 0;
		int ck = 0;
		double x0;
		do {
			NStep++;
			if (NStep > 20) {			//  20ȏJԂ̂Ƃ
				ck = 1;
				break;
			};
			x0 = x;
			x = x0 + (c - f(x0)) / df(x0);
		} while ((Math::Abs(x - x0) >= (acc * (Math::Abs(x) + Math::Abs(x0))))
			&&
			((Math::Abs(x) + Math::Abs(x0)) >= 1.0e-14)
			);

		if (ck == 0) return;

		//   Bisection@ŌvZ

		double L_b = 0.0;
		do {
			L_b -= 1.0;
		} while (f(L_b) >= c);
		double U_b = 0.0;
		do {
			U_b += 1.0;
		} while (f(U_b) <= c);
		Bisection(f, 1.0, c, L_b, U_b, x, acc);
		return;
	}

	class Ct_density {
	public:
		double df;
		double coeff;

		double operator()(double t){
			double v = Math::Pow(1.0 / (1.0 + sqr(t) / df), (df + 1.0) / 2.0);
			return v * coeff;
		}
	};

	class Ccalc_t_for_p {
	public:
		double df, coeff;
		CCum_t Cum_t_distribution;
		Ct_density t_density;

		Ccalc_t_for_p(double vdf) : df(vdf){
			Cum_t_distribution.set_df(df);
			coeff = Cum_t_distribution.coeff;
			t_density.df = df;
			t_density.coeff = coeff;
		}

		double operator()(double p){
			double t = 0.0;
			Newton<CCum_t, Ct_density>(t, p, Cum_t_distribution, t_density, 1.0e-12);
			return t;
		}
	};
}

#endif
